@@ -61,7 +61,7 @@ compute_inference <- function(prior_weights, margliks, is_null = NULL, condition
 
   prior_probs <- prior_weights / sum(prior_weights)
   post_probs  <- unname(bridgesampling::post_prob(margliks, prior_prob = prior_probs))
-  BF          <- inclusion_BF(prior_probs, post_probs, is_null)
+  BF          <- inclusion_BF(prior_probs = prior_probs, margliks = margliks, is_null = is_null)
 
   if(conditional){
     prior_probs <- ifelse(is_null, 0, prior_weights) / sum(prior_weights[!is_null])
@@ -73,8 +73,10 @@ compute_inference <- function(prior_weights, margliks, is_null = NULL, condition
     post_probs  = post_probs,
     BF          = BF
   )
+
   attr(output, "is_null")     <- is_null
   attr(output, "conditional") <- conditional
+  class(output) <- c(class(output), "inference")
 
   return(output)
 }
@@ -93,18 +95,28 @@ ensemble_inference <- function(model_list, parameters, is_null_list, conditional
 
 
   # extract the object
-  margliks   <- sapply(model_list, function(m)m[["marglik"]][["logml"]])
-  prior_weights <- sapply(model_list, function(m)m[["prior_weights"]])
+  margliks      <- sapply(model_list, function(m) m[["marglik"]][["logml"]])
+  prior_weights <- sapply(model_list, function(m) m[["prior_weights"]])
 
   out <- list()
 
   for(p in seq_along(parameters)){
 
     # prepare parameter specific values
-    temp_parameter <- parameters[p]
-    temp_is_null   <- is_null_list[[p]]
+    out[[parameters[p]]] <- compute_inference(prior_weights = prior_weights, margliks = margliks, is_null = is_null_list[[p]], conditional = conditional)
+
+    # add parameter names
+    parameter_name    <- parameters[p]
+    formula_parameter <- unique(unlist(lapply(model_list, function(m) attr(attr(m[["fit"]], "prior_list")[[parameters[p]]], "parameter"))))
+
+    if(!is.null(unlist(formula_parameter))){
+      parameter_name <- gsub(paste0(formula_parameter, "_"), paste0("(", formula_parameter, ") "), parameter_name)
+      parameter_name <- gsub("__xXx__", ":", parameter_name)
+      class(out[[parameters[p]]]) <- c(class(out[[parameters[p]]]), "inference.formula")
+      attr(out[[parameters[p]]], "formula_parameter")  <- formula_parameter
+    }
+    attr(out[[parameters[p]]], "parameter_name")  <- parameter_name
 
-    out[[temp_parameter]] <- compute_inference(prior_weights = prior_weights, margliks = margliks, is_null = temp_is_null, conditional = conditional)
   }
 
   attr(out, "conditional") <- conditional
@@ -127,7 +139,7 @@ models_inference   <- function(model_list){
   incl_BF     <- sapply(seq_along(model_list), function(i){
     is_null <- rep(TRUE, length(model_list))
     is_null[i] <- FALSE
-    return(inclusion_BF(prior_probs, post_probs, is_null))
+    return(inclusion_BF(prior_probs = prior_probs, margliks = margliks, is_null = is_null))
   })
 
   for(i in seq_along(model_list)){
@@ -170,21 +182,22 @@ mix_posteriors <- function(model_list, parameters, is_null_list, conditional = F
   check_char(parameters, "parameters", check_length = FALSE)
   check_list(is_null_list, "is_null_list", check_length = length(parameters))
   check_real(seed, "seed", allow_NULL = TRUE)
-  sapply(model_list, function(m)check_list(m, "model_list:model", check_names = c("fit", "marglik", "priors", "prior_weights"), all_objects = TRUE, allow_other = TRUE))
-  if(!all(sapply(model_list, function(m)inherits(m[["fit"]], what = "runjags")) | sapply(model_list, function(m)inherits(m[["fit"]], what = "rstan")) | sapply(model_list, function(m)inherits(m[["fit"]], what = "null_model"))))
+  sapply(model_list, function(m)check_list(m, "model_list:model", check_names = c("fit", "marglik", "prior_weights"), all_objects = TRUE, allow_other = TRUE))
+  if(!all(sapply(model_list, function(m) inherits(m[["fit"]], what = "runjags")) | sapply(model_list, function(m)inherits(m[["fit"]], what = "rstan")) | sapply(model_list, function(m)inherits(m[["fit"]], what = "null_model"))))
     stop("model_list:fit must contain 'runjags' or 'rstan' models")
-  if(!all(sapply(model_list, function(m)inherits(m[["marglik"]], what = "bridge"))))
+  if(!all(sapply(model_list, function(m) inherits(m[["marglik"]], what = "bridge"))))
     stop("model_list:marglik must contain 'bridgesampling' marginal likelihoods")
-  if(!all(unlist(sapply(model_list, function(m)sapply(m[["priors"]], function(p)is.prior(p))))))
+  if(!all(unlist(sapply(model_list, function(m) sapply(attr(m[["fit"]], "prior_list"), function(p) is.prior(p))))))
     stop("model_list:priors must contain 'BayesTools' priors")
-  sapply(model_list, function(m)check_real(m[["prior_weights"]], "model_list:prior_weights", lower = 0))
+  sapply(model_list, function(m) check_real(m[["prior_weights"]], "model_list:prior_weights", lower = 0))
 
 
   # extract the object
-  fits       <- lapply(model_list, function(m)m[["fit"]])
-  margliks   <- sapply(model_list, function(m)m[["marglik"]][["logml"]])
-  priors     <- lapply(model_list, function(m)m[["priors"]])
-  prior_weights <- sapply(model_list, function(m)m[["prior_weights"]])
+  fits           <- lapply(model_list, function(m) m[["fit"]])
+  margliks       <- sapply(model_list, function(m) m[["marglik"]][["logml"]])
+  priors         <- lapply(model_list, function(m) attr(m[["fit"]], "prior_list"))
+  formula_priors <- lapply(model_list, function(m) m[["formula_priors"]])
+  prior_weights  <- sapply(model_list, function(m) m[["prior_weights"]])
 
   inference  <- ensemble_inference(model_list, parameters, is_null_list, conditional)
   out <- list()
@@ -194,25 +207,10 @@ mix_posteriors <- function(model_list, parameters, is_null_list, conditional = F
     # prepare parameter specific values
     temp_parameter    <- parameters[p]
     temp_inference    <- inference[[temp_parameter]]
-    temp_priors       <- lapply(priors, function(p)p[[temp_parameter]])
+    temp_priors       <- lapply(priors, function(p) p[[temp_parameter]])
 
-    if(all(sapply(temp_priors, is.prior.simple) | sapply(temp_priors, is.prior.point) | sapply(temp_priors, is.null))){
-
-      # replace missing priors with default prior: spike(0)
-      for(i in 1:length(fits)){
-        if(is.null(temp_priors[[i]])){
-          temp_priors[[i]] <- prior("spike", parameters = list("location" = 0))
-        }
-      }
-
-      # replace prior odds with the corresponding prior model odds
-      for(i in seq_along(temp_priors)){
-        temp_priors[[i]][["prior_weights"]] <- temp_inference$prior_probs[i]
-      }
-
-      out[[temp_parameter]] <- .mix_posteriors.simple(fits, temp_priors, temp_parameter, temp_inference$post_probs, seed, n_samples)
-
-    }else if(all(sapply(temp_priors, is.prior.weightfunction) | sapply(temp_priors, is.prior.point) | sapply(temp_priors, is.prior.none) | sapply(temp_priors, is.null))){
+    if(any(sapply(temp_priors, is.prior.weightfunction)) && all(sapply(temp_priors, is.prior.weightfunction) | sapply(temp_priors, is.prior.point) | sapply(temp_priors, is.prior.none) | sapply(temp_priors, is.null))){
+      # weightfunctions:
 
       # replace missing priors with default prior: none
       for(i in 1:length(fits)){
@@ -228,6 +226,65 @@ mix_posteriors <- function(model_list, parameters, is_null_list, conditional = F
 
       out[[temp_parameter]] <- .mix_posteriors.weightfunction(fits, temp_priors, temp_parameter, temp_inference$post_probs, seed, n_samples)
 
+    }else if(any(sapply(temp_priors, is.prior.factor)) && all(sapply(temp_priors, is.prior.factor) | sapply(temp_priors, is.prior.point) | sapply(temp_priors, is.null))){
+      # factor priors
+
+      # replace missing priors with default prior: spike(0)
+      for(i in 1:length(fits)){
+        if(is.null(temp_priors[[i]])){
+          temp_priors[[i]] <- prior("spike", parameters = list("location" = 0))
+        }
+      }
+
+      # replace prior odds with the corresponding prior model odds
+      for(i in seq_along(temp_priors)){
+        temp_priors[[i]][["prior_weights"]] <- temp_inference$prior_probs[i]
+      }
+
+      out[[temp_parameter]] <- .mix_posteriors.factor(fits, temp_priors, temp_parameter, temp_inference$post_probs, seed, n_samples)
+
+    }else if(any(sapply(temp_priors, is.prior.vector)) && all(sapply(temp_priors, is.prior.vector) | sapply(temp_priors, is.prior.point) | sapply(temp_priors, is.null))){
+      # vector priors:
+
+      # replace missing priors with default prior: spike(0)
+      for(i in 1:length(fits)){
+        if(is.null(temp_priors[[i]])){
+          temp_priors[[i]] <- prior("spike", parameters = list("location" = 0))
+        }
+      }
+
+      # replace prior odds with the corresponding prior model odds
+      for(i in seq_along(temp_priors)){
+        temp_priors[[i]][["prior_weights"]] <- temp_inference$prior_probs[i]
+      }
+
+      out[[temp_parameter]] <- .mix_posteriors.vector(fits, temp_priors, temp_parameter, temp_inference$post_probs, seed, n_samples)
+
+    }else if(all(sapply(temp_priors, is.prior.simple) | sapply(temp_priors, is.prior.point) | sapply(temp_priors, is.null))){
+      # simple priors:
+
+      # replace missing priors with default prior: spike(0)
+      for(i in 1:length(fits)){
+        if(is.null(temp_priors[[i]])){
+          temp_priors[[i]] <- prior("spike", parameters = list("location" = 0))
+        }
+      }
+
+      # replace prior odds with the corresponding prior model odds
+      for(i in seq_along(temp_priors)){
+        temp_priors[[i]][["prior_weights"]] <- temp_inference$prior_probs[i]
+      }
+
+      out[[temp_parameter]] <- .mix_posteriors.simple(fits, temp_priors, temp_parameter, temp_inference$post_probs, seed, n_samples)
+
+    }else{
+      stop("The posterior samples cannot be mixed: unsupported mixture of prior distributions.")
+    }
+
+    # add formula relevant information
+    if(!is.null(unique(unlist(lapply(temp_priors, attr, which = "parameter"))))){
+      class(out[[temp_parameter]]) <- c(class(out[[temp_parameter]]), "mixed_posteriors.formula")
+      attr(out[[temp_parameter]], "formula_parameter")  <- unique(unlist(lapply(temp_priors, attr, which = "parameter")))
     }
 
   }
@@ -314,6 +371,185 @@ mix_posteriors <- function(model_list, parameters, is_null_list, conditional = F
 
   return(samples)
 }
+.mix_posteriors.vector         <- function(fits, priors, parameter, post_probs, seed = NULL, n_samples = 10000){
+
+  # check input
+  check_list(fits, "fits")
+  check_list(priors, "priors", check_length = length(fits))
+  check_char(parameter, "parameter")
+  check_real(post_probs, "post_probs", lower = 0, upper = 1, check_length = length(fits))
+  check_real(seed, "seed", allow_NULL = TRUE)
+  if(!all(sapply(fits, inherits, what = "runjags") | sapply(fits, inherits, what = "rstan") | sapply(fits, inherits, what = "null_model")))
+    stop("'fits' must be a list of 'runjags' or 'rstan' models")
+  if(!all(sapply(priors, is.prior.vector) | sapply(priors, is.prior.point)))
+    stop("'priors' must be a list of vector priors")
+
+  # set seed at the beginning makes sure that the samples of different parameters from the same models retain their correlation
+  if(!is.null(seed)){
+    set.seed(seed)
+  }else{
+    set.seed(1)
+  }
+
+  # prepare output objects
+  K <- unique(sapply(priors[sapply(priors, is.prior.vector)], function(p) p$parameters[["K"]]))
+  if(length(K) != 1)
+    stop("all vector priors must be of the same length")
+
+  samples    <- matrix(nrow = 0, ncol = K)
+  sample_ind <- NULL
+  models_ind <- NULL
+
+  # mix samples
+  for(i in seq_along(fits)[round(post_probs * n_samples) > 1]){
+
+    # obtain posterior samples
+    if(inherits(fits[[i]], "null_model")){
+      # deal with a possibility of completely null model
+      model_samples <- matrix()
+    }else if(inherits(fits[[i]], "runjags")){
+      model_samples <- suppressWarnings(coda::as.mcmc(fits[[i]]))
+      if(!is.matrix(model_samples)){
+        # deal with automatic coercion into a vector in case of a single predictor
+        model_samples <- matrix(model_samples, ncol = 1)
+        colnames(model_samples) <- fits[[i]]$monitor
+      }
+    }else if(inherits(fits[[i]], "rstan")){
+      if(!try(requireNamespace("rstan")))
+        stop("rstan package needs to be installed. Run 'install.packages('rstan')'")
+      model_samples <- rstan::extract(fits[[i]])
+      par_names     <- names(model_samples)
+      par_dims      <- sapply(model_samples, function(s)if(is.matrix(s)) ncol(s) else 1)
+      par_names     <- unlist(sapply(seq_along(par_names), function(p){
+        if(par_dims[p] == 1){
+          return(par_names[p])
+        }else{
+          return(paste0(par_names[p], "[", 1:par_dims[p],"]"))
+        }
+      }))
+      model_samples <- data.frame(do.call(cbind, model_samples))
+      colnames(model_samples) <- par_names
+    }
+
+    # sample indexes
+    temp_ind <- sample(nrow(model_samples), round(n_samples * post_probs[i]), replace = TRUE)
+
+    if(is.prior.point(priors[[i]])){
+      samples <- rbind(samples, matrix(rng(priors[[i]], 1), nrow = length(temp_ind), ncol = K))
+    }else{
+      samples <- rbind(samples, model_samples[temp_ind, paste0(parameter,"[",1:K,"]")])
+    }
+
+    sample_ind <- c(sample_ind, temp_ind)
+    models_ind <- c(models_ind, rep(i, length(temp_ind)))
+  }
+
+  rownames(samples) <- NULL
+  colnames(samples) <- paste0(parameter,"[",1:K,"]")
+  attr(samples, "sample_ind") <- sample_ind
+  attr(samples, "models_ind") <- models_ind
+  attr(samples, "parameter")  <- parameter
+  attr(samples, "prior_list") <- priors
+  class(samples) <- c("mixed_posteriors", "mixed_posteriors.vector")
+
+  return(samples)
+}
+.mix_posteriors.factor         <- function(fits, priors, parameter, post_probs, seed = NULL, n_samples = 10000){
+
+  # check input
+  check_list(fits, "fits")
+  check_list(priors, "priors", check_length = length(fits))
+  check_char(parameter, "parameter")
+  check_real(post_probs, "post_probs", lower = 0, upper = 1, check_length = length(fits))
+  check_real(seed, "seed", allow_NULL = TRUE)
+  if(!all(sapply(fits, inherits, what = "runjags") | sapply(fits, inherits, what = "rstan") | sapply(fits, inherits, what = "null_model")))
+    stop("'fits' must be a list of 'runjags' or 'rstan' models")
+  if(!all(sapply(priors, is.prior.factor) | sapply(priors, is.prior.point)))
+    stop("'priors' must be a list of factor priors")
+
+  # check the prior levels
+  levels <- unique(sapply(priors[sapply(priors, is.prior.factor)], function(p) attr(p, "levels")))
+  if(length(levels) != 1)
+    stop("all facotr priors must be of the same number of levels")
+
+  # gather and check compatibility of prior distributions
+  priors_info <- lapply(priors, function(p){
+    if(is.prior.factor(p)){
+      return(list(
+        "levels"      = attr(p, "levels"),
+        "level_names" = attr(p, "level_names"),
+        "interaction" = attr(p, "interaction"),
+        "treatment"   = is.prior.dummy(p),
+        "orthonormal" = is.prior.orthonormal(p)
+      ))
+    }else{
+      return(FALSE)
+    }
+  })
+  priors_info <- priors_info[!sapply(priors_info, isFALSE)]
+  if(length(priors_info) >= 2 && any(!unlist(lapply(priors_info, function(i) all.equal(i, priors_info[[1]]))))){
+    stop("non-matching orthonormal prior specifications")
+  }
+  priors_info <- priors_info[[1]]
+
+  if(all(sapply(priors[sapply(priors, is.prior.factor)], is.prior.dummy))){
+
+    if((levels - 1) == 1){
+
+      samples <- .mix_posteriors.simple(fits, priors, parameter, post_probs, seed, n_samples)
+
+      sample_ind <- attr(samples, "sample_ind")
+      models_ind <- attr(samples, "models_ind")
+
+      samples <- matrix(samples, ncol = 1)
+
+    }else{
+
+      # keep the same seed across levels
+      if(is.null(seed)){
+        seed <- sample(666666, 1)
+      }
+
+      samples <- lapply(1:(levels - 1), function(i) .mix_posteriors.simple(fits, priors, paste0(parameter, "[", i, "]"), post_probs, seed, n_samples))
+
+      sample_ind <- attr(samples[[1]], "sample_ind")
+      models_ind <- attr(samples[[1]], "models_ind")
+
+      samples <- do.call(cbind, samples)
+
+    }
+
+    rownames(samples) <- NULL
+    colnames(samples) <- paste0(parameter,"[",priors_info$level_names[-1],"]")
+    attr(samples, "sample_ind") <- sample_ind
+    attr(samples, "models_ind") <- models_ind
+    attr(samples, "parameter")  <- parameter
+    attr(samples, "prior_list") <- priors
+    class(samples) <- c("mixed_posteriors", "mixed_posteriors.factor", "mixed_posteriors.vector")
+
+  }else if(all(sapply(priors[sapply(priors, is.prior.factor)], is.prior.orthonormal))){
+
+    for(i in seq_along(priors)){
+      if(is.prior.factor(priors[[i]])){
+        priors[[i]]$parameters[["K"]] <- levels - 1
+      }
+    }
+
+    samples <- .mix_posteriors.vector(fits, priors, parameter, post_probs, seed, n_samples)
+    class(samples) <- c(class(samples), "mixed_posteriors.factor")
+
+  }else{
+    stop("all factor priors must be of either 'treatment' or 'orthonormal' type")
+  }
+
+  attr(samples, "levels")      <- priors_info[["levels"]]
+  attr(samples, "level_names") <- priors_info[["level_names"]]
+  attr(samples, "interaction") <- priors_info[["interaction"]]
+  attr(samples, "treatment")   <- priors_info[["treatment"]]
+  attr(samples, "orthonormal") <- priors_info[["orthonormal"]]
+
+  return(samples)
+}
 .mix_posteriors.weightfunction <- function(fits, priors, parameter, post_probs, seed = NULL, n_samples = 10000){
 
   # check input
@@ -402,26 +638,29 @@ mix_posteriors <- function(model_list, parameters, is_null_list, conditional = F
 }
 
 
-
 #' @title Compute inclusion Bayes factors
 #'
-#' @description Computes inclusion Bayes factors based on prior and posterior
-#' model probabilities and indicator whether the models represent the null or
-#' alternative hypothesis
+#' @description Computes inclusion Bayes factors based on prior model probabilities,
+#' posterior model probabilities (or marginal likelihoods), and indicator whether
+#' the models represent the null or alternative hypothesis.
 #'
 #' @param prior_probs vector of prior model probabilities
 #' @param post_probs vector of posterior model probabilities
+#' @param margliks vector of marginal likelihoods.
 #' @param is_null logical vector of indicators whether the model corresponds
 #' to the null or alternative hypothesis (or an integer vector indexing models
 #' corresponding to the null hypothesis)
 #'
+#' @details Supplying \code{margliks} as the input is preferred since it is better at dealing with
+#' under/overflow (posterior probabilities are very close to either 0 or 1). In case that both the
+#' \code{post_probs} and \code{margliks} are supplied, the results are based on \code{margliks}.
+#'
 #' @return \code{inclusion_BF} returns a Bayes factor.
 #'
 #' @export
-inclusion_BF     <- function(prior_probs, post_probs, is_null){
+inclusion_BF         <- function(prior_probs, post_probs, margliks, is_null){
+
 
-  check_real(prior_probs, "prior_probs", lower = 0, upper = 1, check_length = 0)
-  check_real(post_probs,  "post_probs", lower = 0, upper = 1, check_length = length(prior_probs))
   if(is.numeric(is_null)){
     check_int(is_null, "is_null", lower = 1, upper = length(prior_probs), check_length = 0)
     is_null <- c(1:length(prior_probs)) %in% is_null
@@ -431,9 +670,30 @@ inclusion_BF     <- function(prior_probs, post_probs, is_null){
     stop("'is_null' argument must be either logical vector, integer vector, or NULL.")
   }
 
-  if(all(!is_null) | isTRUE(all.equal(sum(post_probs[!is_null]), 1)) | isTRUE(all.equal(sum(prior_probs[!is_null]), 1))){
+  # deal with all null or alternative scenarios
+  if(all(!is_null)){
     return(Inf)
-  }else if(all(is_null) | isTRUE(all.equal(sum(post_probs[is_null]), 1)) | isTRUE(all.equal(sum(prior_probs[is_null]), 1))){
+  }else if(all(is_null)){
+    return(0)
+  }
+
+  if(!missing(prior_probs) && !missing(margliks)){
+    return(.inclusion_BF.margliks(prior_probs = prior_probs, margliks = margliks, is_null = is_null))
+  }else if(!missing(prior_probs) && !missing(post_probs)){
+    return(.inclusion_BF.probs(prior_probs = prior_probs, post_probs = post_probs, is_null = is_null))
+  }else{
+    stop("'prior_probs' and either 'post_probs' or 'marglik' must be specified.")
+  }
+}
+
+.inclusion_BF.probs    <- function(prior_probs, post_probs, is_null){
+
+  check_real(prior_probs, "prior_probs", lower = 0, upper = 1, check_length = 0)
+  check_real(post_probs,  "post_probs", lower = 0, upper = 1, check_length = length(prior_probs))
+
+  if(isTRUE(all.equal(sum(post_probs[!is_null]), 1)) | isTRUE(all.equal(sum(prior_probs[!is_null]), 1))){
+    return(Inf)
+  }else if(isTRUE(all.equal(sum(post_probs[is_null]), 1)) | isTRUE(all.equal(sum(prior_probs[is_null]), 1))){
     return(0)
   }else{
     return(
@@ -442,6 +702,19 @@ inclusion_BF     <- function(prior_probs, post_probs, is_null){
     )
   }
 }
+.inclusion_BF.margliks <- function(prior_probs, margliks, is_null){
+
+  check_real(prior_probs, "prior_probs", lower = 0, upper = 1, check_length = 0)
+  check_real(margliks,  "margliks", check_length = length(prior_probs))
+
+  # center the margliks around the mean for higher precision
+  margliks <- margliks - mean(margliks)
+
+  return(
+    (sum(exp(margliks[!is_null]) * prior_probs[!is_null]) / sum(exp(margliks[is_null]) * prior_probs[is_null])) /
+      (sum(prior_probs[!is_null]) / sum(prior_probs[is_null]))
+  )
+}
 
 
 #' @title Create coefficient mapping between multiple weightfunctions
